package org.richfaces.sandbox.coordinatesGrid;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceDataModel;
import org.ajax4jsf.model.SequenceRange;
import org.richfaces.component.event.CoordinatesSelectionEvent;
import org.richfaces.component.model.SquareRange;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import java.io.IOException;
import java.util.AbstractList;

public class SeatSelector {

    private Integer rows = 10;
    private Integer cols = 20;
    private Integer firstRow = 0;
    private Integer firstCol = 0;
    private Seat[][] seats = new Seat[rows][cols];
    private DataModel dataModel = new CustomDataModel();

    public SeatSelector() {

        final Seat seat = new Seat();
        seat.setEnabled(true);
        seat.setRow(3);
        seat.setSeatNr(5);
        seats[3][4] = seat;
    }

    public Integer getRows() {
        return rows;
    }

    public void setRows(Integer rows) {
        this.rows = rows;
        resizeSeats();
    }

    public Integer getCols() {
        return cols;
    }

    public void setCols(Integer cols) {
        this.cols = cols;
        resizeSeats();
    }

    public Integer getFirstRow() {
        return firstRow;
    }

    public void setFirstRow(Integer row) {
        this.firstRow = row;
    }

    public Integer getFirstCol() {
        return firstCol;
    }

    public void setFirstCol(Integer col) {
        this.firstCol = col;
    }

    public DataModel getDataModel() {
        return dataModel;
    }

    public void itemsSelected(CoordinatesSelectionEvent e) {
        for (int y = e.getStartY(); y <= e.getEndY(); y++) {
            for (int x = e.getStartX(); x <= e.getEndX(); x++) {
                if (seats[y][x] == null) {
                    Seat seat = new Seat();
                    seat.setRow(y);
                    seat.setSeatNr(x);
                    seat.setEnabled(true);
                    seats[y][x] = seat;
                } else {
                    seats[y][x] = null;
                }
            }
        }
        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(e.toString()));
    }

    public void itemsReserved(CoordinatesSelectionEvent e) {
        for (int y = e.getStartY(); y <= e.getEndY(); y++) {
            for (int x = e.getStartX(); x <= e.getEndX(); x++) {
                if (seats[y][x] != null) {
                    seats[y][x].setEnabled(!seats[y][x].isEnabled());
                }
            }
        }
    }

    private void resizeSeats() {
        Seat[][] newSeats = new Seat[rows][cols];
        for (int y = 0; y < Math.min(seats.length, newSeats.length); y++) {
            System.arraycopy(seats[y], 0, newSeats[y], 0, Math.min(seats[y].length, newSeats[y].length));
        }
        seats = newSeats;
    }

    private class CustomDataModel extends SequenceDataModel {

        public CustomDataModel() {
            super(new ListDataModel(new AbstractList<Seat>() {

                @Override
                public Seat get(int index) {
                    int y = index / cols;
                    int x = index % cols;
                    return seats[y][x];
                }

                @Override
                public int size() {
                    return cols * rows;
                }
            }));
        }

        @Override
        public boolean isRowAvailable() {
            return super.isRowAvailable() && getRowData() != null;
        }

        @Override
        public void walk(FacesContext context, DataVisitor visitor, Range range, Object argument) throws IOException {
            SquareRange squareRange = null;
            if (range instanceof SquareRange) {
                squareRange = (SquareRange) range;
            }
            if (squareRange == null) {
                super.walk(context, visitor, new SequenceRange(0, -1), argument);
            } else {
                for (int y = squareRange.getStartY(); y <= squareRange.getEndY(); y++) {
                    for (int x = squareRange.getStartX(); x <= squareRange.getEndX(); x++) {
                        if (0 <= y && y < seats.length && 0 <= x && x < seats[y].length && seats[y][x] != null) {
                            visitor.process(context, y * cols + x, argument);
                        }
                    }
                }
            }
        }
    }
}
