package org.richfaces.sandbox.ui.watermark;

import com.gargoylesoftware.htmlunit.WebClient;
import org.apache.cactus.ServletTestCase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.jsfunit.jsfsession.JSFClientSession;
import org.jboss.jsfunit.jsfsession.JSFSession;

public class WatermarkIT extends ServletTestCase {

    private static Log logger = LogFactory.getLog(WatermarkIT.class);

    public void testAjaxReRender() throws Exception {
        JSFSession jsfSession = new JSFSession("/");
        JSFClientSession client = jsfSession.getJSFClientSession();
        logger.info("Text value: " + client.getElement("text").getAttribute("value"));
        logger.info("Watermarked value: " + client.getElement("watermarked").getAttribute("value"));
        assertEquals(client.getElement("text").getAttribute("value"), client.getElement("watermarked").getAttribute("value"));
        final String text = "Johnny Bravo";
        logger.info("Setting text: " + text);
        int initialJobs = ((WebClient) jsfSession.getWebClient()).waitForBackgroundJavaScript(1000);
        logger.info("Initial jobs: " + initialJobs);
        client.setValue("text", text);
        /**
         * It is very important to wait for those background javascript jobs (ajax request and ui update) before retriving data from
         * client and making assertions.
         */
        int jobs;
        long startTime = System.currentTimeMillis();
        do {
            jobs = ((WebClient) jsfSession.getWebClient()).waitForBackgroundJavaScript(1000);
            logger.info("Number of background JavaScript jobs still running: " + jobs);
            logger.info("Watermarked value: " + client.getElement("watermarked").getAttribute("value"));
            if (startTime + 10000 < System.currentTimeMillis()) {
                throw new RuntimeException("Number of JavaScript jobs doesn't drop to initial level for 10000 seconds. It's memory leak in your JavaScript rather then request taking so long!");
            }
        } while (jobs > initialJobs);
        logger.info("Number of background JavaScript jobs still running: " + jobs);
        assertEquals(text, jsfSession.getJSFServerSession().getManagedBeanValue("#{watermarkBean.text}"));
        assertEquals(null, jsfSession.getJSFServerSession().getManagedBeanValue("#{watermarkBean.watermarkedInput}"));
        logger.info("Text value: " + client.getElement("text").getAttribute("value"));
        logger.info("Watermarked value: " + client.getElement("watermarked").getAttribute("value"));
        assertEquals(text, client.getElement("text").getAttribute("value"));
        assertEquals(text, client.getElement("watermarked").getAttribute("value"));
    }

}
