package org.richfaces.component;

import org.ajax4jsf.component.AjaxActionComponent;
import org.ajax4jsf.component.AjaxComponent;
import org.ajax4jsf.context.AjaxContext;
import org.ajax4jsf.event.AjaxEvent;
import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.renderkit.RendererUtils;
import org.richfaces.component.event.CoordinatesGridEventsProducer;
import org.richfaces.component.event.CoordinatesRefreshEvent;
import org.richfaces.component.event.CoordinatesSelectionEvent;
import org.richfaces.component.event.CoordinatesSelectionListener;
import org.richfaces.component.model.SquareRange;

import javax.el.ELContext;
import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;
import javax.faces.model.ArrayDataModel;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.ResultDataModel;
import javax.faces.model.ResultSetDataModel;
import javax.faces.model.ScalarDataModel;
import javax.servlet.jsp.jstl.sql.Result;
import java.io.IOException;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class UICoordinatesGrid extends UIComponentBase implements CoordinatesGridEventsProducer, AjaxComponent {

    public static final String COMPONENT_TYPE = "org.richfaces.CoordinatesGrid";
    public static final String COMPONENT_FAMILY = "org.richfaces.CoordinatesGrid";
    private DataModel model;

    public abstract Object getValue();

    public abstract void setValue(Object value);

    public abstract String getVar();

    public abstract void setVar(String var);

    public abstract String getRowIndexVar();

    public abstract void setRowIndexVar(String rowIndexVar);

    public abstract String getColIndexVar();

    public abstract void setColIndexVar(String var);

    public abstract String getRowHeaderText();

    public abstract void setRowHeaderText(String rowHeaderText);

    public abstract String getColHeaderText();

    public abstract void setColHeaderText(String colHeaderText);

    public abstract String getColHeaderPosition();

    public abstract void setColHeaderPosition(String colHeaderPosition);

    public abstract String getRowHeaderPosition();

    public abstract void setRowHeaderPosition(String rowHeaderPosition);

    public abstract String getSwitchType();

    public abstract void setSwitchType(String switchType);

    public abstract String getWidgetVar();

    public abstract void setWidgetVar(String widgetVar);

    public abstract String getOnclick();

    public abstract void setOnclick(String onclick);

    public abstract String getOnbeforeselect();

    public abstract void setOnbeforeselect(String onbeforeselect);

    public abstract String getOnselect();

    public abstract void setOnselect(String onselect);

    public abstract Integer getRows();

    public abstract void setRows(Integer rows);

    public abstract Integer getCols();

    public abstract void setCols(Integer cols);

    public abstract boolean isSelectable();

    public abstract void setSelectable(boolean selectable);

    public abstract MethodBinding getSelectionListener();

    public abstract void setSelectionListener(MethodBinding listener);

    @Override
    public void broadcast(FacesEvent event) throws AbortProcessingException {
        if (event instanceof CoordinatesSelectionEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            CoordinatesSelectionEvent selectionEvent = (CoordinatesSelectionEvent) event;
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getSelectionListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
            try {
                ajaxContext.getResponseDataMap().put("_ajax:coordinatesGridData", getData(selectionEvent.getStartX(), selectionEvent.getStartY(), selectionEvent.getEndX(), selectionEvent.getEndY()));
            } catch (IOException ex) {
                getFacesContext().getExternalContext().log("Cannot get coordinates grid data", ex);
            }

        } else if (event instanceof CoordinatesRefreshEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            CoordinatesRefreshEvent refreshEvent = (CoordinatesRefreshEvent) event;
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            try {
                ajaxContext.getResponseDataMap().put("_ajax:coordinatesGridData", getData(refreshEvent.getStartX(), refreshEvent.getStartY(), refreshEvent.getEndX(), refreshEvent.getEndY()));
            } catch (IOException ex) {
                getFacesContext().getExternalContext().log("Cannot get coordinates grid data", ex);
            }

        } else if (event instanceof AjaxEvent) {
            FacesContext context = getFacesContext();
            // complete re-Render fields. AjaxEvent deliver before render
            // response.
            AjaxContext.getCurrentInstance(context).addRegionsFromComponent(this);
            // Put data for send in response
            Object data = getData();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
            if (null != data) {
                ajaxContext.setResponseData(data);
            }
            String focus = getFocus();
            if (null != focus) {
                // search for component in tree.
                // XXX - use more pourful search, as in h:outputLabel
                // component.
                UIComponent focusComponent = RendererUtils.getInstance().
                    findComponentFor(this, focus);
                if (null != focusComponent) {
                    focus = focusComponent.getClientId(context);
                }
                ajaxContext.getResponseDataMap().put(AjaxActionComponent.FOCUS_DATA_ID, focus);
            }
            ajaxContext.setOncomplete(getOncomplete());
        }
    }

    public void addCoordinatesSelectionListener(CoordinatesSelectionListener listener) {
        addFacesListener(listener);
    }

    public void removeCoordinatesSelectionListener(CoordinatesSelectionListener listener) {
        removeFacesListener(listener);
    }

    public CoordinatesSelectionListener[] getCoordinatesSelectionListeners() {
        return (CoordinatesSelectionListener[]) getFacesListeners(CoordinatesSelectionListener.class);
    }

    protected DataModel getDataModel() {
        // Return any previously cached DataModel instance
        if (this.model != null) {
            return (model);
        }

        // Synthesize a DataModel around our current value if possible
        Object current = getValue();
        if (current == null) {
            setDataModel(new ListDataModel(Collections.EMPTY_LIST));
        } else if (current instanceof DataModel) {
            setDataModel((DataModel) current);
        } else if (current instanceof List) {
            setDataModel(new ListDataModel((List) current));
        } else if (Object[].class.isAssignableFrom(current.getClass())) {
            setDataModel(new ArrayDataModel((Object[]) current));
        } else if (current instanceof ResultSet) {
            setDataModel(new ResultSetDataModel((ResultSet) current));
        } else if (current instanceof Result) {
            setDataModel(new ResultDataModel((Result) current));
        } else {
            setDataModel(new ScalarDataModel(current));
        }
        return (model);

    }

    public void setDataModel(DataModel model) {
        this.model = model;
    }

    public List<List<Object>> getData(int startX, int startY, int endX, int endY) throws IOException {
        ELContext elContext = getFacesContext().getELContext();
        ValueExpression valueExpression = getFacesContext().getApplication().getExpressionFactory().createValueExpression(getFacesContext().getELContext(), "#{" + getVar() + "}", Object.class);
        List<List<Object>> data = new ArrayList<List<Object>>();
        DataModel dataModel = getDataModel();
        List<Object> elements = new ArrayList<Object>();
        if (dataModel instanceof ExtendedDataModel) {
            final List<Object> keys = new ArrayList<Object>();
            DataVisitor visitor = new DataVisitor() {

                public void process(FacesContext context, Object rowKey, Object argument) throws IOException {
                    keys.add(rowKey);
                }
            };
            ((ExtendedDataModel) dataModel).walk(getFacesContext(), visitor, new SquareRange(startX, startY, endX, endY), null);
            for (Object key : keys) {
                ((ExtendedDataModel) dataModel).setRowKey(key);
                if (dataModel.isRowAvailable()) {
                    elements.add(dataModel.getRowData());
                }
            }
        } else {
            for (int i = 0; i < dataModel.getRowCount(); i++) {
                dataModel.setRowIndex(i);
                if (dataModel.isRowAvailable()) {
                    elements.add(dataModel.getRowData());
                }
            }
        }

        for (Object element : elements) {
            valueExpression.setValue(elContext, element);
            List<Object> firstDataElement = new ArrayList<Object>();
            for (UIComponent child : getChildren()) {
                if (child instanceof UICoordinatesGridItem) {
                    UICoordinatesGridItem item = (UICoordinatesGridItem) child;
                    if (!item.isRendered()) {
                        continue;
                    }
                    firstDataElement.add(item.getX());
                    firstDataElement.add(item.getY());
                    String text = item.getText();
                    if (text == null) {
                        text = "";
                    }
                    firstDataElement.add(text.trim());
                    String styleClass = item.getStyleClass();
                    if (styleClass == null) {
                        styleClass = "";
                    }
                    firstDataElement.add(styleClass.trim());
                    data.add(firstDataElement);
                }
            }
        }
        valueExpression.setValue(elContext, null);
        return data;
    }

    public List<Map<String, Object>> getRowHeadersData() throws IOException {
        ELContext elContext = getFacesContext().getELContext();
        ValueExpression valueExpression = getFacesContext().getApplication().getExpressionFactory().createValueExpression(getFacesContext().getELContext(), "#{" + getRowIndexVar() + "}", Object.class);
        List<Map<String, Object>> data = new ArrayList<Map<String, Object>>();
        for (int i = 0; i < getRows(); i++) {
            if (getRowIndexVar() != null) {
                valueExpression.setValue(elContext, i);
            }
            Map<String, Object> row = new HashMap<String, Object>();
            data.add(row);
            row.put("text", getRowHeaderText());
        }
        if (getRowIndexVar() != null) {
            valueExpression.setValue(elContext, null);
        }
        return data;
    }

    public List<Map<String, Object>> getColHeadersData() throws IOException {
        ELContext elContext = getFacesContext().getELContext();
        ValueExpression valueExpression = getFacesContext().getApplication().getExpressionFactory().createValueExpression(getFacesContext().getELContext(), "#{" + getColIndexVar() + "}", Object.class);
        List<Map<String, Object>> data = new ArrayList<Map<String, Object>>();
        for (int i = 0; i < getCols(); i++) {
            if (getColIndexVar() != null) {
                valueExpression.setValue(elContext, i);
            }
            Map<String, Object> col = new HashMap<String, Object>();
            data.add(col);
            col.put("text", getColHeaderText());
        }
        if (getColIndexVar() != null) {
            valueExpression.setValue(elContext, null);
        }
        return data;
    }
}
