if (window.RichFaces == null) {
    window.RichFaces = {};
}
RichFaces.CoordinatesGrid = function(targetId, options, submitEventFunction) {

    var startX;
    var startY;
    var endX;
    var endY;
    var selecting = false;
    this.target = document.getElementById(targetId);
    var _this = this;
    this.options = jQuery.extend({}, this.options, options);
    if (this.target.component != null && this.target.component.destroy != null) {
        this.target.component.destroy();
    }
    this.target.component = this;

    var getY = function(o) {
        if (o == null || o.parentNode == null || o.parentNode.rowIndex == null) {
            throw "HTMLTableCellElement expected";
        }
        return o.parentNode.rowIndex;
    };
    var getX = function(o) {
        if (o == null || o.cellIndex == null) {
            throw "HTMLTableCellElement expected";
        }
        return o.cellIndex;
    };
    /**
     * Updates element on the grid with data from initial or ajax request.
     * @param data array of data where: 0: x, 1: y, 2: text, 3: css class
     */
    var updateElementFromData = function(data) {
        var element = _this.getElementAt(data[0], data[1]);
        if (element != null) {
            _this.updateElement(element, {innerHTML:data[2],className:data[3]});
        }
        return element;
    };
    var updateElementsWithRequestData = function(request, _startX, _startY, _endX, _endY) {
        /** Server component should respond with data of elements matching selection */
        var elements = request.getJSON('_ajax:coordinatesGridData');
        if (elements != undefined) {
            var elementsFromUpdate = [];
            jQuery.each(elements, function() {
                var element = updateElementFromData(this);
                if (element != null) {
                    elementsFromUpdate.push(element);
                }
            });
            jQuery.each(_this.getElements(_startX, _startY, _endX, _endY), function() {
                var visitedCell = this;
                var exists = false;
                jQuery.each(elementsFromUpdate, function() {
                    if (this == visitedCell) {
                        exists = true;
                    }
                });
                if (!exists) {
                    this.className = "";
                    this.innerHTML = "";
                }
            });
        }
    };
    var mouseDownHanlder = function() {
        if (selecting) {
            mouseUpHanlder.apply(this);
        } else if (options.selectable) {
            startX = getX(this);
            startY = getY(this);
            selecting = true;
        }
    };
    var mouseUpHanlder = function() {
        if (!selecting) {
            return;
        }
        endX = getX(this);
        endY = getY(this);
        selecting = false;
        highlightSelection();
        if (_this.options.onbeforeselect != null) {
            var result;
            if (typeof _this.options.onbeforeselect == "function") {
                result = _this.options.onbeforeselect(_this, _startX, _startY, _endX, _endY);
            } else {
                result = RichFaces.CoordinatesGrid.eval("(function(){" + _this.options.onbeforeselect + "})()", {
                    'startX':startX,
                    'startY':startY,
                    'endX':endX,
                    'endY':endY,
                    'component':_this
                });
            }
            if (result === false) {
                return;
            }
        }
        /**
         * endX and endY will be reset before ajax response comes.
         */
        var _endX = endX;
        var _endY = endY;
        var _startX = startX;
        var _startY = startY;
        /** Notify server side */
        if (submitEventFunction != null) {
            submitEventFunction({}, "select",
                    _this.indexToCoordX(startX), _this.indexToCoordY(startY), _this.indexToCoordX(endX), _this.indexToCoordY(endY),
                    function(request, event, data) {
                        updateElementsWithRequestData(request, _startX, _startY, _endX, _endY);

                        if (_this.options.onselect != null) {
                            if (typeof _this.options.onselect == "function") {
                                _this.options.onselect(_this, _startX, _startY, _endX, _endY);
                            } else {
                                RichFaces.CoordinatesGrid.eval("(function(){" + _this.options.onselect + "})()", {
                                    'startX':_startX,
                                    'startY':_startY,
                                    'endX':_endX,
                                    'endY':_endY,
                                    'component':_this
                                });
                            }
                        }
                        delete _endX,_endY,_startX,_startY;
                    });
        }
        endX = -1;
        endY = -1;
        highlightSelection();
    };
    var mouseOverHandler = function() {
        if (!selecting) {
            return;
        }
        endX = getX(this);
        endY = getY(this);
        highlightSelection();
    };
    var highlightSelection = function() {
        jQuery("td", _this.target).each(function() {
            var x = getX(this);
            var y = getY(this);
            if (startX <= x && x <= endX && startY <= y && y <= endY) {
                jQuery(this).addClass(_this.options.highlightClass);
            } else {
                jQuery(this).removeClass(_this.options.highlightClass);
            }
        });
    };
    var clickHandler = function() {
        if (_this.options.onclick != null) {
            if (typeof _this.options.onclick == "function") {
                _this.options.onclick(_this, getX(this), getY(this));
            } else {
                RichFaces.CoordinatesGrid.eval("(function(){" + _this.options.onclick + "})()", {
                    'x':getX(this),
                    'y':getY(this),
                    'component':_this
                });
            }
        }
    };
    this.getElements = function(startX, startY, endX, endY) {
        var elements = [];
        jQuery("td", _this.target).each(function() {
            var x = getX(this);
            var y = getY(this);
            if (startX <= x && x <= endX && startY <= y && y <= endY) {
                elements.push(this);
            }
        });
        return elements;
    };
    this.refresh = function(_startX, _startY, _endX, _endY) {
        if (_startX == null && _startY == null && _endX == null && _endY == null) {
            _startX = 0;
            _startY = 0;
            _endX = this.options.cols;
            _endY = this.options.rows;
        }
        submitEventFunction({}, "refresh", _startX, _startY, _endX, _endY, function(request, event, data) {
            updateElementsWithRequestData(request, _startX, _startY, _endX, _endY);
            delete _endX,_endY,_startX,_startY;
        });
    };
    /**
     * @param element HTMLTableCellElement
     * @param options options used to extend element, i.e. {className:'important'}
     **/
    this.updateElement = function(element, options) {
        if (options.className) {
            options.className = "rich-table-cell " + options.className;
        }
        jQuery.extend(element, options);
    };
    this.getElementAt = function(x, y) {
        return table[0].rows[y + (_this.isTopColHeadersVisible() ? 1 : 0)].cells[x + (_this.isLeftRowHeadersVisible() ? 1 : 0)];
    };
    /**
     * Create grid.
     */
    var table = jQuery("<table></table>");
    (function() {
        table.appendTo(_this.target);
        var cell;
        /**
         * Function used for appending headers without filling them with text
         */
        var appendHeaders = function() {
            var row = jQuery("<tr class='rich-table-subheader'></tr>");
            row.appendTo(table);
            row.addClass(_this.options.colHeaderClass);
            if (_this.options.rowHeaders != null) {

                if (_this.options.rowHeaderPosition == 'both' || _this.options.rowHeaderPosition == 'left') {
                    var cell = jQuery("<th class='rich-table-subheadercell'></th>");
                    cell.appendTo(row);
                }
                for (var x = 0; x < _this.options.cols; x++) {
                    cell = jQuery("<th class='rich-table-subheadercell'></th>");
                    cell.appendTo(row);
                }
                if (_this.options.rowHeaderPosition == 'both' || _this.options.rowHeaderPosition == 'right') {
                    cell = jQuery("<th class='rich-table-subheadercell'></th>");
                    cell.appendTo(row);
                }
            }
        };
        /** Append top column headers */
        if (_this.isTopColHeadersVisible()) {
            appendHeaders()
        }
        for (var y = 0; y < _this.options.rows; y++) {
            var row = jQuery("<tr class='rich-table-row'></tr>");
            row.appendTo(table);
            /** Append left row headers */
            if (_this.isLeftRowHeadersVisible()) {
                cell = jQuery("<th class='rich-table-cell'></th>");
                cell.appendTo(row);
                cell.addClass(_this.options.rowHeaderClass);
            }
            /** Append regular cells */
            for (var x = 0; x < _this.options.cols; x++) {
                cell = jQuery("<td class='rich-table-cell'></td>");
                cell.appendTo(row);
                cell.mouseup(mouseUpHanlder);
                cell.mousedown(mouseDownHanlder);
                cell.mouseover(mouseOverHandler);
                cell.click(clickHandler);
            }
            /** Append right row headers */
            if (_this.isRightRowHeadersVisible()) {
                cell = jQuery("<th class='rich-table-cell'></th>");
                cell.appendTo(row);
                cell.addClass(_this.options.rowHeaderClass);
            }
        }
        /** Append bottom column headers */
        if (_this.isBottomColHeadersVisible()) {
            appendHeaders()
        }
        /** Customize headers - className and text */
        jQuery.each(_this.options.rowHeaders, function(index) {
            var text = this.text;
            delete this.text;
            if (text != null) {
                this.innerHTML = text;
            }
            if (_this.isLeftRowHeadersVisible()) {
                _this.updateElement(_this.getElementAt(-1, index), this);
            }
            if (_this.isRightRowHeadersVisible()) {
                _this.updateElement(_this.getElementAt(_this.options.cols, index), this);
            }
        });
        jQuery.each(_this.options.colHeaders, function(index) {
            var text = this.text;
            delete this.text;
            if (text != null) {
                this.innerHTML = text;
            }
            if (_this.isTopColHeadersVisible()) {
                _this.updateElement(_this.getElementAt(index, -1), this);
            }
            if (_this.isBottomColHeadersVisible()) {
                _this.updateElement(_this.getElementAt(index, _this.options.rows), this);
            }
        });
        /** Customize regular cell */
        if (_this.options.elements != null) {
            jQuery.each(_this.options.elements, function() {
                updateElementFromData(this)
            });
        }
        /** Disable text selection */
        if (options.selectable) {
            table.attr('unselectable', 'on').css('MozUserSelect', 'none').bind('selectstart.ui', function() {
                return false;
            });
        }
    })();
}
        ;
RichFaces.CoordinatesGrid.prototype.options = {
    rows: 10,
    cols: 10,
    highlightClass: 'highlight',
    colHeaderPosition: 'bottom',
    rowHeaderPosition: 'left'
};
RichFaces.CoordinatesGrid.prototype.select = function(func) {
    this.options.onselect = func;
};
RichFaces.CoordinatesGrid.prototype.destroy = function() {
    jQuery("table", this.target).remove();
    delete this.target.component;
};
RichFaces.CoordinatesGrid.prototype.isLeftRowHeadersVisible = function() {
    return this.options.rowHeaders != null && (this.options.rowHeaderPosition == 'both' || this.options.rowHeaderPosition == 'left');
};
RichFaces.CoordinatesGrid.prototype.isRightRowHeadersVisible = function() {
    return this.options.rowHeaders != null && (this.options.rowHeaderPosition == 'both' || this.options.rowHeaderPosition == 'right');
};
RichFaces.CoordinatesGrid.prototype.isTopColHeadersVisible = function() {
    return this.options.colHeaders != null && (this.options.colHeaderPosition == 'both' || this.options.colHeaderPosition == 'top');
};
RichFaces.CoordinatesGrid.prototype.isBottomColHeadersVisible = function() {
    return this.options.colHeaders != null && (this.options.colHeaderPosition == 'both' || this.options.colHeaderPosition == 'bottom');
};
RichFaces.CoordinatesGrid.prototype.indexToCoordX = function(index) {
    return index - (this.isLeftRowHeadersVisible() ? 1 : 0);
};
RichFaces.CoordinatesGrid.prototype.indexToCoordY = function(index) {
    return index - (this.isTopColHeadersVisible() ? 1 : 0);
};
/**
 * This function evaluates code in template with object in ScopeChain.
 * This is usefull if you need to evaluate code that uses member names
 * that colide with external names that the code refers to.
 * There is almost exact method in utils.js called Richfaces.eval,
 * but it swallows exception thrown during evaluation, which makes debugging
 * hard.
 */
RichFaces.CoordinatesGrid.eval = function(template, object) {
    var value = '';
    with (object) {
        value = eval(template);
    }
    return value;
};
