package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSLiteral;
import org.ajax4jsf.javascript.JSObject;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.renderkit.RendererUtils;
import org.ajax4jsf.resource.InternetResource;
import org.richfaces.component.UIGmap;
import org.richfaces.component.UIGmapMarker;

import javax.faces.FacesException;
import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class GmapMarkerRenderer extends HeaderResourcesRendererBase {

    private static final Map<String, Object> DEFAULTS;

    /**
     * Following defaults are be used by addOptionIfSetAndNotDefault
     */
    static {
        Map<String, Object> defaults = new HashMap<String, Object>();
        defaults.put("autoPan", UIGmapMarker.DEFAULT_AUTO_PAN);
        defaults.put("hide", UIGmapMarker.DEFAULT_HIDE);
        defaults.put("bounceGravity", UIGmapMarker.DEFAULT_BOUNCE_GRAVITY);
        defaults.put("icon", "");
        defaults.put("zIndexProcess", "");
        defaults.put("title", "");
        defaults.put("iconWidth", "");
        defaults.put("iconHeight", "");
        defaults.put("onclick", "");
        defaults.put("dragCrossMove", UIGmapMarker.DEFAULT_DRAG_CROSS_MOVE);

        DEFAULTS = Collections.unmodifiableMap(defaults);
    }

    private final InternetResource[] scripts = {
            getResource("script/gmap.js"),
            getResource("/org/richfaces/renderkit/html/scripts/richfaces.gmapmarker.js")
    };

    @Override
    protected InternetResource[] getScripts() {
        return scripts;
    }

    @Override
    protected Class<? extends UIComponent> getComponentClass() {
        return UIGmapMarker.class;
    }

    @Override
    protected void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
        UIGmapMarker marker = (UIGmapMarker) component;
        Map<String, Object> options = new HashMap<String, Object>();
        if (marker.getIcon() != null) {
            options.put("icon", new JSObject("GIcon", new JSLiteral("G_DEFAULT_ICON"), marker.getIcon()));
        }
        addOptionIfSetAndNotDefault("clickable", marker.isClickable(), options);
        addOptionIfSetAndNotDefault("draggable", marker.isDraggable(), options);
        addOptionIfSetAndNotDefault("bouncy", marker.isBouncy(), options);
        addOptionIfSetAndNotDefault("autoPan", marker.isAutoPan(), options);
        addOptionIfSetAndNotDefault("hide", marker.isHide(), options);
        addOptionIfSetAndNotDefault("bounceGravity", marker.getBounceGravity(), options);
        addOptionIfSetAndNotDefault("icon", marker.getIcon(), options);
        addOptionIfSetAndNotDefault("title", marker.getTitle(), options);
        addOptionIfSetAndNotDefault("dragCrossMove", marker.isDragCrossMove(), options);
        addOptionIfSetAndNotDefault("iconWidth", marker.getIconWidth(), options);
        addOptionIfSetAndNotDefault("iconHeight", marker.getIconHeight(), options);
        addOptionIfSetAndNotDefault("onclick", marker.getOnclick(), options);
        final UIGmap map = marker.getParentMap();
        final Map<String, Object> mapAttributes = map.getAttributes();
        final Object gmapVar = mapAttributes.get("gmapVar");
        if (gmapVar == null) {
            throw new FacesException("Component " + map.getId() + " must have gmapVar attribute set.");
        }
        final UIComponent infoWindow = marker.getFacet("infoWindow");
        String infoWindowClientId = null;
        if (infoWindow != null) {
            writer.startElement(RendererUtils.HTML.DIV_ELEM, component);
            writer.writeAttribute(RendererUtils.HTML.style_ATTRIBUTE, "display:none", null);
            writer.startElement(RendererUtils.HTML.DIV_ELEM, component);
            infoWindowClientId = marker.getClientId(context) + NamingContainer.SEPARATOR_CHAR + "iW";
            writer.writeAttribute(RendererUtils.HTML.id_ATTRIBUTE, infoWindowClientId, "id");
            infoWindow.encodeAll(context);
            writer.endElement(RendererUtils.HTML.DIV_ELEM);
            writer.endElement(RendererUtils.HTML.DIV_ELEM);
        }
        getUtils().writeScript(context, component,
                new JSFunction("RichFaces.addGmapMarker", gmapVar, marker.getLatitude(), marker.getLongitude(), options, infoWindowClientId)
        );
        Object oninit = mapAttributes.get("oninit");
        if (oninit == null) {
            oninit = "";
        }
        if (!oninit.toString().contains("RichFaces.handleGmapInit")) {
            mapAttributes.put("oninit", oninit + ";" + new JSFunction("RichFaces.handleGmapInit", gmapVar));
        }

    }

    protected void addOptionIfSetAndNotDefault(String optionName, Object value, Map<String, Object> options) {
        if (value != null && !"".equals(value) && !value.equals(DEFAULTS.get(optionName))) {
            options.put(optionName, value);
        }
    }
}
