package org.richfaces.renderkit.html;

import org.ajax4jsf.context.AjaxContext;
import org.ajax4jsf.javascript.ScriptUtils;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.util.InputUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.richfaces.component.UIImageSelectTool;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import java.awt.*;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

public class ImageSelectToolRendererBase extends HeaderResourcesRendererBase {

    private static Log logger = LogFactory.getLog(ImageSelectToolRendererBase.class);

    protected Class<? extends UIComponent> getComponentClass() {
        return UIImageSelectTool.class;
    }

    @Override
    protected void doDecode(FacesContext context, UIComponent component) {
        super.doDecode(context, component);
        UIImageSelectTool uiImageSelectTool;

        if (component instanceof UIImageSelectTool) {
            uiImageSelectTool = (UIImageSelectTool) component;
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("No decoding necessary since the component " + component.getId() +
                    " is not an instance or a sub class of UIInplaceInput");
            }
            return;
        }
        if (InputUtils.isDisabled(uiImageSelectTool) || InputUtils.isReadOnly(uiImageSelectTool)) {
            if (logger.isDebugEnabled()) {
                logger.debug(("No decoding necessary since the component " + component.getId() + " is disabled"));
            }
            return;
        }
        String inputId = getInputId(context, uiImageSelectTool);
        if (inputId == null) {
            throw new NullPointerException("component client id is null");
        }
        Map<String, String> request = context.getExternalContext().getRequestParameterMap();

        String newValue = request.get(inputId);
        if (newValue != null) {
            uiImageSelectTool.setSubmittedValue(newValue);
        }
    }

    public String getInputId(FacesContext context, UIImageSelectTool component) {
        return component.getClientId(context) + ":input";

    }

    public Converter getConverter(FacesContext context, UIImageSelectTool component) {
        Converter converter = component.getConverter();
        if (converter == null) {
            converter = new RectangleConverter();
        }
        return converter;
    }

    @Override
    public Object getConvertedValue(FacesContext context, UIComponent component, Object o) throws ConverterException {
        UIImageSelectTool uiImageSelectTool = (UIImageSelectTool) component;
        Converter converter = getConverter(context, uiImageSelectTool);
        String valueString = (String) o;
        return converter.getAsObject(context, component, valueString);

    }

    protected String getValueAsString(FacesContext context, UIComponent component) throws IOException {
        UIImageSelectTool uiImageSelectTool = (UIImageSelectTool) component;
        String valueString = (String) uiImageSelectTool.getSubmittedValue();
        if (valueString == null) {
            Object value = uiImageSelectTool.getValue();
            if (value != null) {
                Converter converter = getConverter(context, uiImageSelectTool);
                valueString = converter.getAsString(context, component, value);
            }
        }
        return valueString;
    }

    public void writeInitFunction(FacesContext context, UIImageSelectTool component) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        if (component.getVar() != null) {
            writer.write("var " + component.getVar() + "=");
        }
        Map<String, Object> options = new HashMap<String, Object>();
        String forAttribute = component.getFor();
        UIComponent forComp;
        String forClientId = null;
        if (forAttribute == null || forAttribute.length() == 0) {
            forComp = component;
            while ((forComp = forComp.getParent()) != null) {
                if (forComp instanceof javax.faces.component.UIGraphic) {
                    break;
                }
            }
            /**
             * We are not interested in UIViewRoot
             */
            if (forComp.getParent() == null) {
                forComp = null;
            }
        } else {
            forComp = getUtils().findComponentFor(context, component.getParent(), forAttribute);
        }
        if (forComp == null) {
            throw new FacesException("could not find target for croptool " + component.getId());
        }
        forClientId = forComp.getClientId(context);
        addOptionIfSet("ajaxRequest", AjaxContext.getCurrentInstance().isAjaxRequest(), options);
        addOptionIfSet("onchange", component.getOnchange(), options);
        addOptionIfSet("onselect", component.getOnselect(), options);
        Rectangle rect = (Rectangle) component.getValue();
        if (rect != null) {
            Map<String, Object> selection = new HashMap<String, Object>();
            selection.put("x", rect.x);
            selection.put("y", rect.y);
            selection.put("width", rect.width);
            selection.put("height", rect.height);
            addOptionIfSet("selection", selection, options);
        }
        addOptionIfSet("maxWidth", component.getMaxWidth(), options);
        addOptionIfSet("maxHeight", component.getMaxHeight(), options);
        addOptionIfSet("minWidth", component.getMinWidth(), options);
        addOptionIfSet("minHeight", component.getMinHeight(), options);
        addOptionIfSet("aspectRatio", component.getAspectRatio(), options);
        addOptionIfSet("backgroundColor", component.getBackgroundColor(), options);
        addOptionIfSet("backgroundOpacity", component.getBackgroundOpacity(), options);
        addOptionIfSet("trueSizeWidth", component.getTrueSizeWidth(), options);
        addOptionIfSet("trueSizeHeight", component.getTrueSizeHeight(), options);
        writer.write("new RichFaces.ImageSelectTool(");
        writer.write(ScriptUtils.toScript(getUtils().clientId(context, component)));
        writer.write(",");
        writer.write(ScriptUtils.toScript(getInputId(context, component)));
        writer.write(",");
        writer.write(ScriptUtils.toScript(forClientId));
        writer.write(",");
        writer.write(ScriptUtils.toScript(options));
        writer.write(");");
    }

    private void addOptionIfSet(String optionName, Object value, Map<String, Object> options) {
        if (value != null && value != "") {
            options.put(optionName, value);
        }
    }

    private class RectangleConverter implements Converter {

        public Object getAsObject(FacesContext facesContext, UIComponent component, String s) {
            StringTokenizer tokenizer = new StringTokenizer(s, ";");
            try {
                Integer x = Integer.parseInt(tokenizer.nextToken());
                Integer y = Integer.parseInt(tokenizer.nextToken());
                Integer w = Integer.parseInt(tokenizer.nextToken());
                Integer h = Integer.parseInt(tokenizer.nextToken());
                return new Rectangle(x, y, w, h);
            } catch (Exception e) {
                throw new ConverterException("conversion failure; allowed pattern X;Y;W;H", e);
            }
        }

        public String getAsString(FacesContext facesContext, UIComponent component, Object o) {
            if (!(o instanceof Rectangle)) {
                throw new ConverterException(o + " is not instance of java.awt.Rectangle");
            }
            Rectangle r = (Rectangle) o;
            return r.x + ";" + r.y + ";" + r.width + ";" + r.height;
        }
    }
}
