package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.renderkit.RendererUtils;
import org.ajax4jsf.renderkit.compiler.TemplateContext;
import org.ajax4jsf.resource.InternetResource;
import org.ajax4jsf.resource.InternetResourceBuilder;
import org.richfaces.component.UILightbox;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class LightboxRenderer extends HeaderResourcesRendererBase {

    private static final Map<String, Object> DEFAULTS;

    /**
     * Following defaults are be used by addOptionIfSetAndNotDefault
     */
    static {
        Map<String, Object> defaults = new HashMap<String, Object>();
        defaults.put("overlayBgColor", UILightbox.DEFAULT_OVERLAY_BG_COLOR);
        defaults.put("overlayOpacity", UILightbox.DEFAULT_OVERLAY_OPACITY);
        defaults.put("containerBorderSize", UILightbox.DEFAULT_CONTAINER_BORDER_SIZE);
        defaults.put("containerResizeSpeed", UILightbox.DEFAULT_CONTAINER_RESIZE_SPEED);
        defaults.put("fixedNavigation", UILightbox.DEFAULT_FIXED_NAVIGATION);
        defaults.put("keyToClose", UILightbox.DEFAULT_KEY_TO_CLOSE);
        defaults.put("keyToNext", UILightbox.DEFAULT_KEY_TO_NEXT);
        defaults.put("keyToPrev", UILightbox.DEFAULT_KEY_TO_PREV);
        defaults.put("txtImage", UILightbox.DEFAULT_TXT_IMAGE);
        defaults.put("txtOf", UILightbox.DEFAULT_TXT_OF);

        DEFAULTS = Collections.unmodifiableMap(defaults);
    }

    private final InternetResource[] scripts = {
        getResource("/org/richfaces/renderkit/html/scripts/jquery/jquery.js"),
        getResource("/org/richfaces/renderkit/html/scripts/jquery.lightbox.js"),
        getResource("/org/richfaces/renderkit/html/scripts/richfaces.lightbox.js")
    };

    private final InternetResource[] styles = {
        getResource("/org/richfaces/renderkit/html/css/jquery.lightbox.css"),
    };

    @Override
    protected InternetResource[] getScripts() {
        return scripts;
    }

    @Override
    protected InternetResource[] getStyles() {
        return styles;
    }

    @Override
    protected Class<? extends UIComponent> getComponentClass() {
        return UILightbox.class;
    }

    @Override
    protected void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
        UILightbox lightbox = (UILightbox) component;
        final HashMap<String, Object> options = new HashMap<String, Object>();
        addOptionIfSetAndOrDefault("imageBlank", lightbox.getImageBlank(), getURL(context, "/org/richfaces/renderkit/html/images/lightbox-blank.gif"), options);
        addOptionIfSetAndOrDefault("imageLoading", lightbox.getImageLoading(), getURL(context, "/org/richfaces/renderkit/html/images/lightbox-ico-loading.gif"), options);
        addOptionIfSetAndOrDefault("imageBtnNext", lightbox.getImageBtnNext(), getURL(context, "/org/richfaces/renderkit/html/images/lightbox-btn-next.gif"), options);
        addOptionIfSetAndOrDefault("imageBtnPrev", lightbox.getImageBtnPrev(), getURL(context, "/org/richfaces/renderkit/html/images/lightbox-btn-prev.gif"), options);
        addOptionIfSetAndOrDefault("imageBtnClose", lightbox.getImageBtnClose(), getURL(context, "/org/richfaces/renderkit/html/images/lightbox-btn-close.gif"), options);
        addOptionIfSetAndNotDefault("containerBorderSize", lightbox.getContainerBorderSize(), options);
        addOptionIfSetAndNotDefault("containerResizeSpeed", lightbox.getContainerResizeSpeed(), options);
        addOptionIfSetAndNotDefault("fixedNavigation", lightbox.getFixedNavigation(), options);
        addOptionIfSetAndNotDefault("keyToClose", lightbox.getKeyToClose(), options);
        addOptionIfSetAndNotDefault("keyToNext", lightbox.getKeyToNext(), options);
        addOptionIfSetAndNotDefault("keyToPrev", lightbox.getKeyToPrev(), options);
        addOptionIfSetAndNotDefault("overlayBgColor", lightbox.getOverlayBgColor(), options);
        addOptionIfSetAndNotDefault("overlayOpacity", lightbox.getOverlayOpacity(), options);
        addOptionIfSetAndNotDefault("txtImage", lightbox.getTxtImage(), options);
        addOptionIfSetAndNotDefault("txtOf", lightbox.getTxtOf(), options);
        writer.startElement(RendererUtils.HTML.DIV_ELEM, component);
        writer.writeAttribute(RendererUtils.HTML.id_ATTRIBUTE, getUtils().clientId(context, component), "id");
        getUtils().writeScript(context, component, new JSFunction("RichFaces.Lightbox", lightbox.getSelector(), options));
        writer.endElement(RendererUtils.HTML.DIV_ELEM);
    }

    protected void addOptionIfSetAndNotDefault(String optionName, Object value, Map<String, Object> options) {
        if (value != null && !"".equals(value) && !value.equals(DEFAULTS.get(optionName))) {
            options.put(optionName, value);
        }
    }

    protected void addOptionIfSetAndOrDefault(String optionName, Object value, Object defaultValue, Map<String, Object> options) {
        if (value != null && !"".equals(value)) {
            options.put(optionName, value);
        } else {
            options.put(optionName, defaultValue);
        }
    }

    private String getURL(FacesContext context, String path) {
        return InternetResourceBuilder.getInstance().createResource(null, path).getUri(context, null);
    }
}
