/*
 * JBoss, Home of Professional Open Source
 * Copyright , Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.renderkit.RendererUtils;
import org.ajax4jsf.resource.InternetResource;
import org.richfaces.component.UINotifyStack;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class NotifyStackRenderer extends HeaderResourcesRendererBase {

    private final InternetResource[] scripts = {
        getResource("/org/richfaces/renderkit/html/scripts/jquery/jquery.js"),
        getResource("/org/richfaces/renderkit/html/scripts/jquery.pnotify.js"),
        getResource("/org/richfaces/renderkit/html/scripts/richfaces.notify.js")
    };

    @Override
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        if (!(component instanceof UINotifyStack)) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.startElement(RendererUtils.HTML.SCRIPT_ELEM, null);
        writer.writeText(new JSFunction("RichFaces.NotifyStack.register",
            RendererUtils.getInstance().clientId(context, component),
            getOptions((UINotifyStack) component)
        ), null);
        writer.endElement(RendererUtils.HTML.SCRIPT_ELEM);
    }

    protected Map<String, Object> getOptions(UINotifyStack stack) throws IOException {
        /**
         * Include only attributes that are actually set.
         */
        Map<String, Object> options = new HashMap<String, Object>();
        addOptionIfSet("dir1", stack.getStackDir1(), options);
        addOptionIfSet("dir2", stack.getStackDir2(), options);
        addOptionIfSet("push", stack.getPush(), options);
        return options;
    }

    protected void addOptionIfSet(String optionName, Object value, Map<String, Object> options) {
        if (value != null && !"".equals(value)) {
            options.put(optionName, value);
        }
    }

    @Override
    protected InternetResource[] getScripts() {
        return scripts;
    }

    @Override
    protected Class<? extends UIComponent> getComponentClass() {
        return UINotifyStack.class;
    }
}
