package org.richfaces.component;

import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.component.UISelectItem;
import javax.faces.component.UISelectItems;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

public abstract class UIRadio extends UIOutput {
// ------------------------------ FIELDS ------------------------------

    public static final String COMPONENT_FAMILY = "org.richfaces.Radio";

    public static final String COMPONENT_TYPE = "org.richfaces.Radio";

// -------------------------- STATIC METHODS --------------------------

    /**
     * <p>Return a List of {@link javax.faces.model.SelectItem}
     * instances representing the available options for this component,
     * assembled from the set of {@link javax.faces.component.UISelectItem}
     * and/or {@link javax.faces.component.UISelectItems} components that are
     * direct children of this component.  If there are no such children, an
     * empty <code>Iterator</code> is returned.</p>
     *
     * @param context   The {@link javax.faces.context.FacesContext} for the current request.
     *                  If null, the UISelectItems behavior will not work.
     * @param component the component
     * @return a List of the select items for the specified component
     * @throws IllegalArgumentException if <code>context</code>
     *                                  is <code>null</code>
     */
    public static List<SelectItem> getSelectItems(FacesContext context,
                                                  UIComponent component) {
        if (context == null) {
            throw new IllegalArgumentException("Context param must not be null.");
        }

        ArrayList<SelectItem> list = new ArrayList<SelectItem>();
        for (UIComponent kid : component.getChildren()) {
            if (kid instanceof UISelectItem) {
                UISelectItem item = (UISelectItem) kid;
                Object value = item.getValue();

                if (value == null) {
                    list.add(new SelectItem(item.getItemValue(),
                            item.getItemLabel(),
                            item.getItemDescription(),
                            item.isItemDisabled(),
                            item.isItemEscaped()));
                } else if (value instanceof SelectItem) {
                    list.add((SelectItem) value);
                } else {
                    throw new IllegalArgumentException(
                            "Argument Error: An option for component ''" + component.getId() + "'' was not an instance of javax.faces.model.SelectItem.  Type found: ''" + value.getClass().getName() + "''.");
                }
            } else if (kid instanceof UISelectItems) {
                Object value = ((UISelectItems) kid).getValue();
                if (value instanceof SelectItem) {
                    list.add((SelectItem) value);
                } else if (value instanceof SelectItem[]) {
                    SelectItem[] items = (SelectItem[]) value;
                    // we manually copy the elements so that the list is
                    // modifiable.  Arrays.asList() returns a non-mutable
                    // list.
                    //noinspection ManualArrayToCollectionCopy
                    for (SelectItem item : items) {
                        list.add(item);
                    }
                } else if (value instanceof Collection) {
                    for (Object element : ((Collection) value)) {
                        if (SelectItem.class.isInstance(element)) {
                            list.add((SelectItem) element);
                        } else {
                            throw new IllegalArgumentException(
                                    "Argument Error: An option for component ''" + component.getId() + "'' was not an instance of javax.faces.model.SelectItem.  Type found: ''" + value.getClass().getName() + "''.");
                        }
                    }
                } else if (value instanceof Map) {
                    Map optionMap = (Map) value;
                    for (Object o : optionMap.entrySet()) {
                        Map.Entry entry = (Map.Entry) o;
                        Object key = entry.getKey();
                        Object val = entry.getValue();
                        if (key == null || val == null) {
                            continue;
                        }
                        list.add(new SelectItem(val,
                                key.toString()));
                    }
                } else {
                    throw new IllegalArgumentException("Expected a child component type of ''UISelectItem/UISelectItems'' for component type ''" + component.getFamily() + "(" + component.getId() + ")''.  Found ''" + (value != null ? value.getClass().getName() : "null") + "''.");
                }
            }
        }
        return (list);
    }

// -------------------------- OTHER METHODS --------------------------

    public abstract String getFor();

    public abstract Integer getIndex();

    public SelectItem getSelectItem(FacesContext context, UIComponent targetComponent) {
        final List<SelectItem> list = getSelectItems(context, targetComponent);
        try {
            return list.get(getIndex());
        } catch (IndexOutOfBoundsException e) {
            throw new IllegalArgumentException("Component ''" + getId() + "'' has wrong value of index attribute (" + getIndex() + "). Target component ''" + targetComponent.getId() + "'' has only " + list.size() + " items.");
        }
    }

    public abstract void setFor(String _for);

    public abstract void setIndex(Integer index);
}
