package org.richfaces.component;

import org.ajax4jsf.component.AjaxActionComponent;
import org.ajax4jsf.component.AjaxComponent;
import org.ajax4jsf.context.AjaxContext;
import org.ajax4jsf.event.AjaxEvent;
import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.renderkit.RendererUtils;
import org.richfaces.component.event.*;
import org.richfaces.component.model.DateRange;

import javax.el.ELContext;
import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.FacesEvent;
import javax.faces.model.ArrayDataModel;
import javax.faces.model.DataModel;
import javax.faces.model.ListDataModel;
import javax.faces.model.ResultDataModel;
import javax.faces.model.ResultSetDataModel;
import javax.faces.model.ScalarDataModel;
import javax.servlet.jsp.jstl.sql.Result;
import java.io.IOException;
import java.sql.ResultSet;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

public abstract class UISchedule extends UIComponentBase implements ScheduleCommonViewAttributes, ScheduleListenerEventsProducer, AjaxComponent {
// ------------------------------ FIELDS ------------------------------

    public static final String COMPONENT_FAMILY = "org.richfaces.Schedule";

    public static final String COMPONENT_TYPE = "org.richfaces.Schedule";

    /**
     * Values of switchType attribute
     */
    public static final String SWITCH_TYPE_AJAX = "ajax";

    public static final String SWITCH_TYPE_CLIENT = "client";

    public static final String SWITCH_TYPE_SERVER = "server";

    public static final String VIEW_AGENDA_DAY = "agendaDay";

    public static final String VIEW_AGENDA_WEEK = "agendaWeek";

    public static final String VIEW_BASIC_DAY = "basicDay";

    public static final String VIEW_BASIC_WEEK = "basicWeek";

    /**
     * Values of view attribute.
     */
    public static final String VIEW_MONTH = "month";

    /**
     * Values of weekMode attribute.
     */
    public static final String WEEK_MODE_FIXED = "fixed";

    public static final String WEEK_MODE_LIQUID = "liquid";

    public static final String WEEK_MODE_VARIABLE = "variable";

    public static final boolean DEFAULT_ALL_DAY_DEFAULT = true;

    public static final boolean DEFAULT_ALL_DAY_SLOT = true;

    public static final double DEFAULT_ASPECT_RATIO = 1.35;

    public static final String DEFAULT_AXIS_FORMAT = "h(:mm)tt";

    public static final boolean DEFAULT_DISABLE_DRAGGING = false;

    public static final boolean DEFAULT_DISABLE_RESIZING = false;

    public static final double DEFAULT_DRAG_OPACITY = .3;

    public static final int DEFAULT_DRAG_REVERT_DURATION = 500;

    public static final boolean DEFAULT_EDITABLE = false;

    public static final int DEFAULT_EVENT_MINUTES = 120;

    public static final int DEFAULT_FIRST_DAY = Calendar.SUNDAY;

    public static final int DEFAULT_FIRST_HOUR = 6;

    public static final String DEFAULT_MAX_TIME = "24";

    public static final String DEFAULT_MIN_TIME = "0";

    public static final boolean DEFAULT_RTL = false;

    public static final boolean DEFAULT_SELECTABLE = false;

    public static final boolean DEFAULT_SELECT_HELPER = false;

    public static final boolean DEFAULT_SHOW_WEEKENDS = true;

    public static final int DEFAULT_SLOT_MINUTES = 30;

    public static final String DEFAULT_SWITCH_TYPE = SWITCH_TYPE_AJAX;

    public static final boolean DEFAULT_UNSELECT_AUTO = true;

    public static final String DEFAULT_UNSELECT_CANCEL = "";

    public static final String DEFAULT_VIEW = VIEW_MONTH;

    public static final String DEFAULT_WEEK_MODE = WEEK_MODE_FIXED;

    private DataModel model;

// -------------------------- STATIC METHODS --------------------------

    public static int getFirstDay(UISchedule schedule) {
        Integer firstDay = schedule.getFirstDay();
        return firstDay == null ? UISchedule.DEFAULT_FIRST_DAY : firstDay;
    }

    public static Date getFirstDisplayedDay(UISchedule schedule) {
        Calendar calendar = Calendar.getInstance();
        Date date = schedule.getDate();
        if (date != null) {
            calendar.setTime(date);
        }
        int firstDayOfWeek = getFirstDay(schedule);
        calendar.setFirstDayOfWeek(firstDayOfWeek);
        String view = getView(schedule);
        boolean showWeekends = isShowWeekends(schedule);
        if (VIEW_MONTH.equals(view)) {
            calendar.set(Calendar.DATE, 1);
            if (!showWeekends) {
                int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
                if (dayOfWeek == Calendar.SUNDAY) {
                    calendar.add(Calendar.DATE, 1);
                } else if (dayOfWeek == Calendar.SATURDAY) {
                    calendar.add(Calendar.DATE, 2);
                }
            }
            if (!showWeekends && firstDayOfWeek == Calendar.SUNDAY) {
                firstDayOfWeek++;
            }
            /**
             * Following 1 line is a fix to what i believe is a bug of java.util.Calendar
             */
            calendar.get(Calendar.DAY_OF_WEEK);
            calendar.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);
            return calendar.getTime();
        } else if (VIEW_AGENDA_WEEK.equals(view) || VIEW_BASIC_WEEK.equals(view)) {
            calendar.set(Calendar.DAY_OF_WEEK, firstDayOfWeek);
            if (!showWeekends) {
                int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
                if (dayOfWeek == Calendar.SUNDAY) {
                    calendar.add(Calendar.DATE, 1);
                } else if (dayOfWeek == Calendar.SATURDAY) {
                    calendar.add(Calendar.DATE, 2);
                }
            }
            return calendar.getTime();
        } else if (VIEW_AGENDA_DAY.equals(view) || VIEW_BASIC_DAY.equals(view)) {
            return calendar.getTime();
        } else {
            throw new IllegalStateException("Invalid view attribute: " + view);
        }
    }

    public static Date getLastDisplayedDate(UISchedule schedule) {
        Calendar calendar = Calendar.getInstance();
        int firstDayOfWeek = getFirstDay(schedule);
        String view = getView(schedule);
        boolean showWeekends = isShowWeekends(schedule);
        if (VIEW_MONTH.equals(view)) {
            if (WEEK_MODE_FIXED.equals(getWeekMode(schedule))) {
                calendar.setTime(getFirstDisplayedDay(schedule));
                calendar.add(Calendar.DAY_OF_YEAR, 42);
            } else {
                Date date = schedule.getDate();
                if (date != null) {
                    calendar.setTime(date);
                }

                if (!showWeekends && firstDayOfWeek == Calendar.SUNDAY) {
                    firstDayOfWeek++;
                }
                calendar.setFirstDayOfWeek(firstDayOfWeek);
                calendar.set(Calendar.DATE, calendar.getActualMaximum(Calendar.DATE));
                int dayOfWeek = firstDayOfWeek + 6;
                if (dayOfWeek > Calendar.SATURDAY) {
                    dayOfWeek -= 7;
                }
                /**
                 * Following 1 line is a fix to what i believe is a bug of java.util.Calendar
                 */
                calendar.get(Calendar.DAY_OF_WEEK);
                calendar.set(Calendar.DAY_OF_WEEK, dayOfWeek);
                calendar.add(Calendar.DATE, 1);
            }
            return calendar.getTime();
        } else if (VIEW_AGENDA_WEEK.equals(view) || VIEW_BASIC_WEEK.equals(view)) {
            calendar.setFirstDayOfWeek(firstDayOfWeek);
            calendar.setTime(getFirstDisplayedDay(schedule));
            calendar.add(Calendar.DATE, 7);
            if (!showWeekends) {
                int dayOfWeek = calendar.get(Calendar.DAY_OF_WEEK);
                if (dayOfWeek - 1 == Calendar.SUNDAY) {
                    calendar.add(Calendar.DATE, -2);
                } else if (dayOfWeek - 1 == Calendar.SATURDAY) {
                    calendar.add(Calendar.DATE, -1);
                }
            }
            return calendar.getTime();
        } else if (VIEW_AGENDA_DAY.equals(view) || VIEW_BASIC_DAY.equals(view)) {
            calendar.setTime(getFirstDisplayedDay(schedule));
            calendar.add(Calendar.DATE, 1);
            return calendar.getTime();
        } else {
            throw new IllegalStateException("Invalid view attribute: " + view);
        }
    }

    public static String getView(UISchedule schedule) {
        String view = schedule.getView();
        return view == null ? UISchedule.DEFAULT_VIEW : view;
    }

    public static String getWeekMode(UISchedule schedule) {
        String weekMode = schedule.getWeekMode();
        return weekMode == null ? UISchedule.DEFAULT_WEEK_MODE : weekMode;
    }

    /**
     * Here go static methods for getting value or default value of attributes.
     */
    /**
     * @param schedule
     * @return
     */
    public static boolean isShowWeekends(UISchedule schedule) {
        Boolean showWeekends = schedule.getShowWeekends();
        return showWeekends == null ? UISchedule.DEFAULT_SHOW_WEEKENDS : schedule.getShowWeekends();
    }

// ------------------------ INTERFACE METHODS ------------------------


// --------------------- Interface ScheduleListenerEventsProducer ---------------------

    public void addItemSelectListener(ScheduleItemSelectListener listener) {
        addFacesListener(listener);
    }

    public void removeItemSelectListener(ScheduleItemSelectListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleItemSelectListener[] getItemSelectListeners() {
        return (ScheduleItemSelectListener[]) getFacesListeners(ScheduleItemSelectListener.class);
    }

    public void addItemMoveListener(ScheduleItemMoveListener listener) {
        addFacesListener(listener);
    }

    public void removeItemMoveListener(ScheduleItemMoveListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleItemMoveListener[] getItemMoveListeners() {
        return (ScheduleItemMoveListener[]) getFacesListeners(ScheduleItemMoveListener.class);
    }

    public void addItemResizeListener(ScheduleItemResizeListener listener) {
        addFacesListener(listener);
    }

    public void removeItemResizeListener(ScheduleItemResizeListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleItemResizeListener[] getItemResizeListeners() {
        return (ScheduleItemResizeListener[]) getFacesListeners(ScheduleItemResizeListener.class);
    }

    public void addViewChangeListener(ScheduleViewChangeListener listener) {
        addFacesListener(listener);
    }

    public void removeViewChangeListener(ScheduleViewChangeListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleViewChangeListener[] getViewChangeListeners() {
        return (ScheduleViewChangeListener[]) getFacesListeners(ScheduleViewChangeListener.class);
    }

    public void addDateRangeChangeListener(ScheduleDateRangeChangeListener listener) {
        addFacesListener(listener);
    }

    public void removeDateRangeChangeListener(ScheduleDateRangeChangeListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleDateRangeChangeListener[] getDateRangeChangeListeners() {
        return (ScheduleDateRangeChangeListener[]) getFacesListeners(ScheduleDateRangeChangeListener.class);
    }

    public void addDateRangeSelectListener(ScheduleDateRangeSelectListener listener) {
        addFacesListener(listener);
    }

    public void removeDateRangeSelectListener(ScheduleDateRangeSelectListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleDateRangeSelectListener[] getDateRangeSelectListeners() {
        return (ScheduleDateRangeSelectListener[]) getFacesListeners(ScheduleDateRangeSelectListener.class);
    }

    public void addDateSelectListener(ScheduleDateSelectListener listener) {
        addFacesListener(listener);
    }

    public void removeDateSelectListener(ScheduleDateSelectListener listener) {
        removeFacesListener(listener);
    }

    public ScheduleDateSelectListener[] getDateSelectListeners() {
        return (ScheduleDateSelectListener[]) getFacesListeners(ScheduleDateSelectListener.class);
    }

// -------------------------- OTHER METHODS --------------------------

    /**
     * React on various events.
     * Vetoable events are first broadcasted to listeners bound via EL to
     * component attribtues and then if no veto is raised then to the rest of
     * listeners.
     * In case of non vetoable events the order of broadcast is reverse.
     * Vetoable events are: ScheduleItemMoveEvent, ScheduleItemResizeEvent
     * Non-vetoable events: ScheduleDateRangeChangeEvent, ScheduleItemSelectEvent,
     * ScheduleViewChangeEvent, ScheduleDateSelectEvent,
     * ScheduleDateRangeSelectEvent
     * In case of ScheduleDateRangeChangeEvent new items are returned
     * via response data map of ajaxContext.
     * In case of ScheduleItemMoveEvent and ScheduleItemResizeEvent, the
     * decision if veto was raised is sent back to client via response data map
     * of ajaxContext as boolean.
     *
     * @param event broadcasted event
     * @throws AbortProcessingException if broadcasting of particular event
     *                                  should stop
     */
    @Override
    public void broadcast(FacesEvent event) throws AbortProcessingException {
        if (event instanceof ScheduleDateRangeChangeEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            ScheduleDateRangeChangeEvent calendarAjaxEvent = (ScheduleDateRangeChangeEvent) event;
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getDateRangeChangeListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
            try {
                ajaxContext.getResponseDataMap().put("_ajax:scheduleData", getScheduleData(calendarAjaxEvent.getStartDate(), calendarAjaxEvent.getEndDate()));
            } catch (IOException ex) {
                getFacesContext().getExternalContext().log("Cannot get schedule data", ex);
            }
        } else if (event instanceof ScheduleItemMoveEvent) {
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getItemMoveListener();
            boolean allow = true;
            if (expression != null) {
                Object result = expression.invoke(facesContext, new Object[]{event});
                allow = ((Boolean) result);
            }
            ajaxContext.getResponseDataMap().put("_ajax:scheduleData", allow);
            super.broadcast(event);
            new AjaxEvent(this).queue();
        } else if (event instanceof ScheduleItemResizeEvent) {
            FacesContext facesContext = getFacesContext();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(facesContext);
            MethodBinding expression = getItemResizeListener();
            boolean allow = true;
            if (expression != null) {
                Object result = expression.invoke(facesContext, new Object[]{event});
                allow = ((Boolean) result);
            }
            ajaxContext.getResponseDataMap().put("_ajax:scheduleData", allow);
            super.broadcast(event);
            new AjaxEvent(this).queue();
        } else if (event instanceof ScheduleItemSelectEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getItemSelectListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        } else if (event instanceof ScheduleViewChangeEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getViewChangeListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        } else if (event instanceof ScheduleDateSelectEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getDateSelectListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        } else if (event instanceof ScheduleDateRangeSelectEvent) {
            super.broadcast(event);
            new AjaxEvent(this).queue();
            FacesContext facesContext = getFacesContext();
            MethodBinding expression = getDateRangeSelectListener();
            if (expression != null) {
                expression.invoke(facesContext, new Object[]{event});
            }
        } else if (event instanceof AjaxEvent) {
            FacesContext context = getFacesContext();
            // complete re-Render fields. AjaxEvent deliver before render
            // response.
            AjaxContext.getCurrentInstance(context).addRegionsFromComponent(this);
            // Put data for send in response
            Object data = getData();
            AjaxContext ajaxContext = AjaxContext.getCurrentInstance(context);
            if (null != data) {
                ajaxContext.setResponseData(data);
            }
            String focus = getFocus();
            if (null != focus) {
                // search for component in tree.
                // XXX - use more pourful search, as in h:outputLabel
                // component.
                UIComponent focusComponent = RendererUtils.getInstance().
                    findComponentFor(this, focus);
                if (null != focusComponent) {
                    focus = focusComponent.getClientId(context);
                }
                ajaxContext.getResponseDataMap().put(AjaxActionComponent.FOCUS_DATA_ID, focus);
            }
            ajaxContext.setOncomplete(getOncomplete());
        } else {
            super.broadcast(event);
        }
    }

    public abstract Boolean getAllDayByDefault();

    public abstract Boolean getAllDaySlot();

    public abstract String getAllDayText();

    public abstract Double getAspectRatio();

    public abstract String getAxisFormat();

    public abstract Integer getContentHeight();

    public abstract Date getDate();

    public abstract MethodBinding getDateRangeChangeListener();

    public abstract MethodBinding getDateRangeSelectListener();

    public abstract MethodBinding getDateSelectListener();

    public abstract Integer getDefaultEventMinutes();

    public abstract Boolean getDisableDragging();

    public abstract Boolean getDisableResizing();

    public abstract Integer getDragRevertDuration();

    public abstract Boolean getEditable();

    public abstract Integer getFirstDay();

    public abstract Integer getFirstHour();

    public abstract String getHeaderCenter();

    public abstract String getHeaderLeft();

    public abstract String getHeaderRight();

    public abstract Integer getHeight();

    public abstract Boolean getIsRTL();

    public abstract MethodBinding getItemMoveListener();

    public abstract MethodBinding getItemResizeListener();

    public abstract MethodBinding getItemSelectListener();

    public abstract String getMaxTime();

    public abstract String getMinTime();

    public abstract String getOnbeforedaterangeselect();

    public abstract String getOnbeforedateselect();

    public abstract String getOnbeforeitemdrop();

    public abstract String getOnbeforeitemresize();

    public abstract String getOnbeforeitemselect();

    public abstract String getOndaterangechange();

    public abstract String getOndaterangeselect();

    public abstract String getOndateselect();

    public abstract String getOnitemdragstart();

    public abstract String getOnitemdragstop();

    public abstract String getOnitemdrop();

    public abstract String getOnitemmouseout();

    public abstract String getOnitemmouseover();

    public abstract String getOnitemresize();

    public abstract String getOnitemresizestart();

    public abstract String getOnitemresizestop();

    public abstract String getOnitemselect();

    public abstract String getOnviewchange();

    public abstract String getOnviewdisplay();

    /**
     * Range is [startDate;endDate), which means that start date is inclusive
     * and end date is exclusive.
     *
     * @param startDate
     * @param endDate
     * @return
     * @throws IOException
     */
    public List<Map<String, Object>> getScheduleData(Date startDate, Date endDate) throws IOException {
        /**
         * Locale must be US because this is the format the javascript widget supports
         */
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm", Locale.US);
        format.setLenient(false);
        DataModel dataModel = getDataModel();
        if (dataModel instanceof ExtendedDataModel) {
            DataVisitor visitor = new DataVisitor() {
                //TODO Is this fine? or should we stack rowKeys and not use dataModel later on. I'don't know business rules of extendedDataModel, just used it once to do pagination with underlying EntityQuery from Seam

                public void process(FacesContext context, Object rowKey, Object argument) throws IOException {
                }
            };
            ((ExtendedDataModel) dataModel).walk(getFacesContext(), visitor, new DateRange(startDate, endDate), null);
        }
        ELContext elContext = getFacesContext().getELContext();
        ValueExpression valueExpression = getFacesContext().getApplication().getExpressionFactory().createValueExpression(getFacesContext().getELContext(), "#{" + getVar() + "}", Object.class);
        List<Map<String, Object>> data = new ArrayList<Map<String, Object>>();
        for (int i = 0; i < dataModel.getRowCount(); i++) {
            dataModel.setRowIndex(i);
            valueExpression.setValue(elContext, dataModel.getRowData());
            Map<String, Object> firstDataElement = new HashMap<String, Object>();
// TODO shouldn't we check earlier if there is at most one renderable UIScheduleItem child?
            for (UIComponent child : getChildren()) {
                if (child instanceof UIScheduleItem) {
                    UIScheduleItem item = (UIScheduleItem) child;
                    if (!item.isRendered()) {
                        continue;
                    }
                    firstDataElement.put("id", item.getEventId());
                    firstDataElement.put("title", item.getTitle());
                    if (item.getAllDay() != null) {
                        firstDataElement.put("allDay", item.getAllDay());
                    }
                    firstDataElement.put("start", format.format(item.getStartDate()));
                    if (item.getEndDate() != null) {
                        firstDataElement.put("end", format.format(item.getEndDate()));
                    }
                    if (item.getUrl() != null) {
                        firstDataElement.put("url", item.getUrl());
                    }
                    if (item.getStyleClass() != null) {
                        firstDataElement.put("className", item.getStyleClass());
                    }
                    if (item.getEditable() != null) {
                        firstDataElement.put("editable", item.getEditable());
                    }
                    if (item.getData() != null) {
                        firstDataElement.put("data", item.getData());
                    }
                    data.add(firstDataElement);
                }
            }
        }
        valueExpression.setValue(elContext, null);
        return data;
    }

    public abstract Boolean getSelectHelper();

    public abstract Boolean getSelectable();

    public abstract Boolean getShowWeekends();

    public abstract Integer getSlotMinutes();

    public abstract String getStyleClass();

    public abstract String getSwitchType();

    public abstract Boolean getUnselectAuto();

    public abstract String getUnselectCancel();

    public abstract Object getValue();

    public abstract String getVar();

    public abstract String getView();

    public abstract MethodBinding getViewChangeListener();

    public abstract String getWeekMode();

    public abstract String getWidgetVar();

    public abstract void setAllDayByDefault(Boolean allDayByDefault);

    public abstract void setAllDaySlot(Boolean allDaySlot);

    public abstract void setAllDayText(String allDayText);

    public abstract void setAspectRatio(Double aspectRatio);

    public abstract void setAxisFormat(String axisFormat);

    public abstract void setContentHeight(Integer contentHeight);

    public void setDataModel(DataModel model) {
        this.model = model;
    }

    public abstract void setDate(Date date);

    public abstract void setDateRangeChangeListener(MethodBinding listener);

    public abstract void setDateRangeSelectListener(MethodBinding listener);

    public abstract void setDateSelectListener(MethodBinding listener);

    public abstract void setDefaultEventMinutes(Integer defaultEventMinutes);

    public abstract void setDisableDragging(Boolean disableDragging);

    public abstract void setDisableResizing(Boolean disableResizing);

    public abstract void setDragRevertDuration(Integer dragRevertDuration);

    public abstract void setEditable(Boolean editable);

    public abstract void setFirstDay(Integer firstDay);

    public abstract void setFirstHour(Integer firstHour);

    public abstract void setHeaderCenter(String headerCenter);

    public abstract void setHeaderLeft(String headerLeft);

    public abstract void setHeaderRight(String headerRight);

    public abstract void setHeight(Integer height);

    public abstract void setIsRTL(Boolean isrtl);

    public abstract void setItemMoveListener(MethodBinding listener);

    public abstract void setItemResizeListener(MethodBinding listener);

    public abstract void setItemSelectListener(MethodBinding listener);

    public abstract void setMaxTime(String maxTime);

    public abstract void setMinTime(String minTime);

    public abstract void setOnbeforedaterangeselect(String onbeforedaterangeselect);

    public abstract void setOnbeforedateselect(String onbeforedateselect);

    public abstract void setOnbeforeitemdrop(String onbeforeitemdrop);

    public abstract void setOnbeforeitemresize(String onbeforeitemresize);

    public abstract void setOnbeforeitemselect(String onbeforeitemselect);

    public abstract void setOndaterangechange(String ondaterangechange);

    public abstract void setOndaterangeselect(String ondaterangeselect);

    public abstract void setOndateselect(String ondateselect);

    public abstract void setOnitemdragstart(String onitemdragstop);

    public abstract void setOnitemdragstop(String onitemdragstop);

    public abstract void setOnitemdrop(String onitemdrop);

    public abstract void setOnitemmouseout(String onitemmouseout);

    public abstract void setOnitemmouseover(String onitemmouseover);

    public abstract void setOnitemresize(String onitemresize);

    public abstract void setOnitemresizestart(String onitemresizestart);

    public abstract void setOnitemresizestop(String onitemresizestop);

    public abstract void setOnitemselect(String onitemselect);

    public abstract void setOnviewchange(String onviewchange);

    public abstract void setOnviewdisplay(String onviewdisplay);

    public abstract void setSelectHelper(Boolean selectHelper);

    public abstract void setSelectable(Boolean selectable);

    public abstract void setShowWeekends(Boolean showWeekends);

    public abstract void setSlotMinutes(Integer slotMinutes);

    public abstract void setStyleClass(String styleClass);

    public abstract void setSwitchType(String switchType);

    public abstract void setUnselectAuto(Boolean unselectAuto);

    public abstract void setUnselectCancel(String unselectCancel);

    public abstract void setValue(Object value);

    public abstract void setVar(String value);

    public abstract void setView(String defaultView);

    public abstract void setViewChangeListener(MethodBinding listener);

    public abstract void setWeekMode(String weekMode);

    public abstract void setWidgetVar(String widgetVar);

    protected DataModel getDataModel() {
        // Return any previously cached DataModel instance
        if (this.model != null) {
            return model;
        }

        // Synthesize a DataModel around our current value if possible
        Object current = getValue();
        if (current == null) {
            setDataModel(new ListDataModel(Collections.EMPTY_LIST));
        } else if (current instanceof DataModel) {
            setDataModel((DataModel) current);
        } else if (current instanceof List) {
            setDataModel(new ListDataModel((List) current));
        } else if (Object[].class.isAssignableFrom(current.getClass())) {
            setDataModel(new ArrayDataModel((Object[]) current));
        } else if (current instanceof ResultSet) {
            setDataModel(new ResultSetDataModel((ResultSet) current));
        } else if (current instanceof Result) {
            setDataModel(new ResultDataModel((Result) current));
        } else {
            setDataModel(new ScalarDataModel(current));
        }
        return model;
    }
}
