package org.richfaces.renderkit.html.scripts;

import org.ajax4jsf.resource.ClientScript;
import org.ajax4jsf.resource.ResourceContext;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.faces.application.Application;
import javax.faces.context.FacesContext;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class ScheduleMessages extends ClientScript {
// ------------------------------ FIELDS ------------------------------

    public static final String BUNDLE_NAME = "org.richfaces.component.UIScheduleMessages";

    private static final Log LOG = LogFactory.getLog(ScheduleMessages.class);

    private static final String MESSAGE_KEY_BASE = "org.richfaces.component.UISchedule.";

// ------------------------ INTERFACE METHODS ------------------------


// --------------------- Interface InternetResource ---------------------

    @Override
    public InputStream getResourceAsStream(ResourceContext context) {
//TODO how to access messages from here if user keeps them somewhere else?
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        FacesContext facesContext = FacesContext.getCurrentInstance();
        Application application = facesContext.getApplication();
        StringBuilder out = new StringBuilder();
//        space at the end is crucial if no locale is supported
        out.append("RichFaces.Schedule.prototype.messages=jQuery.extend(RichFaces.Schedule.prototype.messages,{ ");
        Iterator<Locale> supportedLocales = application.getSupportedLocales();
        int localeCount = 0;
        while (supportedLocales.hasNext()) {
            localeCount++;
            Locale locale = supportedLocales.next();
            ResourceBundle applicationBundle = ResourceBundle.getBundle(application.getMessageBundle(), locale, loader);
            ResourceBundle stockBundle = null;
            try {
                stockBundle = ResourceBundle.getBundle(BUNDLE_NAME, locale, loader);
            } catch (MissingResourceException e) {
                LOG.debug("No bundle " + BUNDLE_NAME + " for locale " + locale + " found.");
            }
            String[] months = new String[]{"JANUARY", "FEBRUARY", "MARCH", "APRIL", "MAY", "JUNE", "JULY", "AUGUST",
                    "SEPTEMBER", "OCTOBER", "NOVEMBER", "DECEMBER"};
            String[] days = new String[]{"SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY", "THURSDAY", "FRIDAY", "SATURDAY"};
            out.append("'").append(locale.toString()).append("':{");
            out.append("allDayText:'").append(escape(getMessageFromBundle(MESSAGE_KEY_BASE + "allDay",
                    applicationBundle, stockBundle))).append("',");
            appendArray(out, applicationBundle, stockBundle, "monthNames", "monthNames", months);
            out.append(",");
            appendArray(out, applicationBundle, stockBundle, "monthNamesShort", "monthNamesShort", months);
            out.append(",");
            appendArray(out, applicationBundle, stockBundle, "dayNames", "dayNames", days);
            out.append(",");
            appendArray(out, applicationBundle, stockBundle, "dayNamesShort", "dayNamesShort", days);
            out.append(",");
            appendMap(out, applicationBundle, stockBundle, "buttonText", "buttonTexts", new String[]{"prev", "next",
                    "prevYear", "nextYear", "today", "month", "day", "week"});
            out.append("},");
        }
        if (localeCount > 0) {
            out.delete(out.length() - 1, out.length());
        }
        out.append("})");
        try {
//            TODO where to get encoding from? It should match properties file's encoding, but probably be converted to response encoding
            return new ByteArrayInputStream(out.toString().getBytes(application.getViewHandler()
                    .calculateCharacterEncoding(facesContext)));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean requireFacesContext() {
        return true;
    }

// -------------------------- OTHER METHODS --------------------------

    @Override
    public String getJavaScript() {
        return getClass().getCanonicalName();
    }

    private void appendArray(StringBuilder out, ResourceBundle applicationBundle, ResourceBundle stockBundle,
                             String jsPropertyName, String prefix, String[] keys) {
        String key;
        out.append(jsPropertyName).append(":[");
        for (int i = 0; i < keys.length; i++) {
            key = MESSAGE_KEY_BASE + prefix + "." + keys[i];
            out.append("'").append(escape(getMessageFromBundle(key, applicationBundle, stockBundle))).append("'");
            if (i + 1 < keys.length) {
                out.append(",");
            }
        }
        out.append("]");
    }

    private void appendMap(StringBuilder out, ResourceBundle applicationBundle, ResourceBundle stockBundle,
                           String jsPropertyName, String prefix, String[] keys) {
        String key;
        out.append(jsPropertyName).append(":{");
        for (int i = 0; i < keys.length; i++) {
            key = MESSAGE_KEY_BASE + prefix + "." + keys[i];
            out.append(keys[i]).append(":").append("'").append(escape(
                    getMessageFromBundle(key, applicationBundle, stockBundle))
            ).append("'");
            if (i + 1 < keys.length) {
                out.append(",");
            }
        }
        out.append("}");
    }

    private String escape(String message) {
        return message.replaceAll("'", "\\\\'");
    }

    private String getMessageFromBundle(String key, ResourceBundle applicationBundle, ResourceBundle stockBundle) {
        try {
            return applicationBundle.getString(key);
        } catch (MissingResourceException e) {
            if (stockBundle == null) {
                throw e;
            }
            try {
                return stockBundle.getString(key);
            } catch (MissingResourceException e1) {
                if (LOG.isWarnEnabled()) {
                    LOG.warn("Cannot find resource " + e1.getKey() + " in bundle " + e1.getClassName());
                }
                return "";
            }
        }
    }
}
